package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.WMSFeign;
import com.steerinfo.dil.feign.WmshBoundFeign;
import com.steerinfo.dil.mapper.TmstruckEnfactoryResultMapper;
import com.steerinfo.dil.mapper.TmstruckLoadResultMapper;
import com.steerinfo.dil.model.TmstruckLoadResult;
import com.steerinfo.dil.service.ITmstruckLoadResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

/**
 * TmstruckLoadResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-02 10:38
 * 类描述
 * 修订历史：
 * 日期：2021-09-02
 * 作者：generator
 * 参考：
 * 描述：TmstruckLoadResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckLoadResultService")
public class TmstruckLoadResultServiceImpl implements ITmstruckLoadResultService {

    @Autowired
    private TmstruckLoadResultMapper tmstruckLoadResultMapper;

    @Autowired
    private TmstruckEnfactoryResultMapper tmstruckEnfactoryResultMapper;


    @Autowired
    private UtilsServiceImpl utilsService;

    @Autowired
    private WmshBoundFeign wmshBoundFeign;
    
    @Autowired
    private WMSFeign wmsFeign;



    /**
     * 新增装车实绩
     * 运输订单号 或 运输订单Id
     * 是否清场
     * 装车日期
     * 备注
     * 计量吨位
     * @param map
     * @return
     */
    @Transactional
    public int addLoadResult11(Map<String, Object> map) {
        //通过运输订单号 或 运单Id 查询实绩总表ID  此方法在 进厂作业mapper中
        Integer totalId = tmstruckEnfactoryResultMapper.selectOrderIdByCpAndOn(map);
        // 新增装车实绩
        TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
        //添加主键Id
        tmstruckLoadResult.setResultId(tmstruckLoadResultMapper.selectMaxId());
        //新增计量吨位 装车所装的重量
        BigDecimal resultMeasuredTonnage = DataChange.dataToBigDecimal(map.get("resultMeasuredTonnage"));
        tmstruckLoadResult.setResultMeasuredTonnage(resultMeasuredTonnage);
        tmstruckLoadResult.setStatus(new BigDecimal(0));
        tmstruckLoadResult.setResultTotalId(new BigDecimal(totalId));
        String resultIsclear = (String) (map.get("resultIsclear"));
        tmstruckLoadResult.setResultIsclear(resultIsclear);
        tmstruckLoadResult.setResultLoadStartTime(new Date((long) map.get("resultLoadStartTime")));
        tmstruckLoadResult.setInsertUpdateRemark((String) map.get("insertUpdateRemark"));
        int i = tmstruckLoadResultMapper.insertSelective(tmstruckLoadResult);

        //修改订单状态为执行中 1
        int i1 = utilsService.updateOrderStatus(map, 1);

        //修改运力状态为执行中 1
        int i2 = utilsService.updateCapacityStatus(map, 1);

        //新增港存库出库实绩 并修改实时库存
        BigDecimal portId = tmstruckLoadResultMapper.getPortIdBySendStationId(DataChange.dataToBigDecimal(map.get("loadingId")));
        if(portId != null){
            BigDecimal batchId = tmstruckLoadResultMapper.getBatchIdByOrderId(DataChange.dataToBigDecimal(map.get("orderId")));
            if(batchId != null){
                Map<String, Object> hashMap = new HashMap<>();
                hashMap.put("batchId", batchId);
                hashMap.put("portId", portId);
                hashMap.put("resultTonnage",resultMeasuredTonnage);
                hashMap.put("isClean", resultIsclear);
                wmshBoundFeign.addResult(hashMap);
            }
        }
        return i + i1 + i2;
    }

    /**
     * 添加装货实绩
     * @param map
     * @return
     */
    public int addLoadResult(Map<String, Object> map){
        return 0;
    }




    /**
     * 遗留问题  （销售装车）
     * @param map
     * @return
     */
    public int addLoadResult1(Map<String, Object> map) {
//        String orderNumber = map.get("orderNumber").toString();
        //通过运输订单号查找物资
        List<Map<String, Object>> materialList = tmstruckLoadResultMapper.selectMaterialIdByOrderNumber(map);
        for (int i = 0; i < materialList.size(); i++) {
            Map<String, Object> map3 = materialList.get(i);
            BigDecimal materialId = (BigDecimal) map3.get("materialId");
            //通过运输订单号 或 运单Id 查询实绩总表ID  此方法在 进厂作业mapper中
            Integer totalId = tmstruckEnfactoryResultMapper.selectOrderIdByCpAndOn(map);
            // 新增装车实绩
            TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
            //添加主键Id
            tmstruckLoadResult.setResultId(tmstruckLoadResultMapper.selectMaxId());
            //新增计量吨位 装车所装的重量
            BigDecimal resultMeasuredTonnage = DataChange.dataToBigDecimal(map.get("resultMeasuredTonnage"));
            tmstruckLoadResult.setResultMeasuredTonnage(resultMeasuredTonnage);
            tmstruckLoadResult.setStatus(new BigDecimal(0));
            tmstruckLoadResult.setResultTotalId(new BigDecimal(totalId));
            String resultIsclear = (String) (map.get("resultIsclear"));
            tmstruckLoadResult.setResultIsclear(resultIsclear);
            //根据运输订单号查找运输订单类型
            Integer orderType = tmstruckLoadResultMapper.selectOrderTypeByOrderNumber((String) map.get("orderNumber"));
            int i2 = 0;
            int i1 = 0;
            switch (orderType) {
                case 3:
                    tmstruckLoadResult.setResultLoadStartTime(new Date((long) map.get("resultLoadStartTime")));
                    //修改订单状态为执行中 1
                    i1 = utilsService.updateOrderStatus(map, 1);
                    //修改运力状态为执行中 1
                    i2 = utilsService.updateCapacityStatus(map, 1);
                    //新增出库实绩 并修改实时库存
                    BigDecimal portId = tmstruckLoadResultMapper.getPortIdBySendStationId(DataChange.dataToBigDecimal(map.get("loadingId")));
                    if (portId != null) {
                        BigDecimal batchId = tmstruckLoadResultMapper.getBatchIdByOrderId(DataChange.dataToBigDecimal(map.get("orderId")));
                        if (batchId != null) {
                            HashMap<String, Object> hashMap = new HashMap<>();
                            hashMap.put("batchId", batchId);
                            hashMap.put("portId", portId);
                            hashMap.put("resultTonnage", resultMeasuredTonnage);
                            hashMap.put("isClean", resultIsclear);
                            wmshBoundFeign.addResult(hashMap);
                        }
                    }
                case 1:
                    List list = tmstruckLoadResultMapper.selectResultIdByOrderNumber((String) map.get("orderNumber"));
                    //查询最大id和最小id
                    Integer maxResultId = (Integer) Collections.max(list);
                    Integer minResultId = (Integer) Collections.min(list);
                    //根据最大id查找扫描时间
                    Map map1 = tmstruckLoadResultMapper.selectLoadTimeByResultId(minResultId);
                    Date startdate = (Date) map1.get("loadTime");
                    Integer loaderId = tmstruckLoadResultMapper.selecLoadIdByresultId(minResultId);
                    //根据最小id查找扫描时间
                    Map map2 = tmstruckLoadResultMapper.selectLoadTimeByResultId(maxResultId);
                    Date endDate = (Date) map2.get("loadTime");
                    tmstruckLoadResult.setResultLoadStartTime(startdate);
                    tmstruckLoadResult.setResultLoadEndTime(endDate);
                    //开始时间
                    long startTime = startdate.getTime();
                    long endTime = endDate.getTime();
                    long time = endTime - startTime;
                    long l = time / 1000;
                    tmstruckLoadResult.setResultLoadDuration(new BigDecimal(l));
                    tmstruckLoadResult.setLoaderId(new BigDecimal(loaderId));
                    //新增出库实绩
                    wmsFeign.insertOutBoundResult(map);

            }
            tmstruckLoadResult.setMaterialId(materialId);
            tmstruckLoadResult.setInsertUpdateRemark((String) map.get("insertUpdateRemark"));
            int i3 = tmstruckLoadResultMapper.insertSelective(tmstruckLoadResult);
            return i3 + i1 + i2;
        }

        return 6;
    }
    
    /**
     * 修改装车作业
     * @param tmstruckLoadResult
     * @return
     */
    @Override
    public int updateLoadResult(TmstruckLoadResult tmstruckLoadResult) {
        return tmstruckLoadResultMapper.updateByPrimaryKeySelective(tmstruckLoadResult);
    }

    /**
     * 通过ID查询装车实绩
     * @param resultId
     * @return
     */
    @Override
    public List<Map<String, Object>> getLoadResultById(Integer resultId) {
        return tmstruckLoadResultMapper.getLoadResultById(resultId);
    }

    /**
     * 逻辑删除装车实绩 状态 1
     * @param resultId
     * @return
     */
    @Override
    public int deleteLoadResult(Integer resultId) {
        TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
        tmstruckLoadResult.setResultId(new BigDecimal(resultId));
        tmstruckLoadResult.setStatus(new BigDecimal(1));
        return tmstruckLoadResultMapper.updateByPrimaryKeySelective(tmstruckLoadResult);
    }

    /**
     * 修改装车作业点 多个装车点随机分配给多个装车实绩
     * @param mapList
     * @return
     */
    @Override
    public int updateLoadingId(List<Map<String, Object>> mapList) {
        int i = 0;
        //获取所有装车实绩ID
        for (Map<String, Object> map : mapList) {
            TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
            //通过传过来的物资ID（一个订单中物资ID唯一） 和订单Id 寻找装车实绩ID
            Integer resultIdList = tmstruckLoadResultMapper.getLoadResultIdList(map);
            tmstruckLoadResult.setResultId(new BigDecimal(resultIdList));
            //获取算出来的装车点
            tmstruckLoadResult.setLoadingId(DataChange.dataToBigDecimal(map.get("loadingId")));
            i += tmstruckLoadResultMapper.updateByPrimaryKeySelective(tmstruckLoadResult);
        }
        return i;
    }

    /**
     * 查询销售合同装车实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> selectLoadResultForSale(Map<String, Object> map) {
        return tmstruckLoadResultMapper.selectLoadResultForSale(map);
    }

    /**
     * 新增销售装车实绩(虚拟)
     *
     * @param map {orderId 运输订单Id}
     * @return
     */
    public int addSaleLoadResult(Map<String, Object> map){
//        String orderNumber = (String) map.get("orderNumber");
        List<Map<String, Object>> materialList = tmstruckLoadResultMapper.selectMaterialIdByOrderNumber(map);
        int i = 0;
        for (Map<String, Object> materialMap : materialList) {
            TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
            //添加物资ID
            tmstruckLoadResult.setMaterialId(DataChange.dataToBigDecimal(materialMap.get("materialId")));
            //添加总实绩ID
            Integer totalId = tmstruckEnfactoryResultMapper.selectOrderIdByCpAndOn(map);
            tmstruckLoadResult.setResultTotalId(new BigDecimal(totalId));
            //添加主键
            tmstruckLoadResult.setResultId(tmstruckLoadResultMapper.selectMaxId());
            i += tmstruckLoadResultMapper.insertSelective(tmstruckLoadResult);
        }
        return i;
    }

    /**
     * 查询所有装车实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllLoadResult(Map<String, Object> map) {
        Integer orderType = (Integer) map.get("orderTypee");
        switch (orderType){
            case 1:

                break;
            case 2:

                break;
            case 3:

                break;
            case 4:

                break;
            case 5:

                break;
            case 6:

                break;
            case 7:

                break;
            case 8:
                return tmstruckLoadResultMapper.getCGJKLoadResult(map);
            case 9:

                break;
            case 10:

                break;
            case 11:
        }
        return null;
    }



}
